﻿//----------------------------------------------------
// EditTitleBlockForm.cs (c) 2019 by Enes Siljak
//----------------------------------------------------

using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Text.RegularExpressions;

namespace XenAlati
{
    public partial class EditTitleBlockForm : Form
    {
        private TextBox textBox;
        private ListBox listCounters;
        private Label label1;
        private Button btnAddNumericCounter;
        private Button btnOK;
        int countCounters = 0;

        public EditTitleBlockForm(List<string> items)
        {
            InitializeComponent();
            countCounters = 0;
            SetItems(items);
        }

        public void SetItems(List<string> items)
        {
            foreach (string it in items)
                textBox.AppendText( string.Format("{0} \r\n", it)  ); 
        }

        public string GetText()
        {
            string result = RemoveComments( textBox.Text.TrimEnd() ) + "!";
            foreach (object o in listCounters.Items)
                result += string.Format("{0} \r\n", o.ToString() );

            return result.TrimEnd();

        }

        string RemoveComments(string inputtext)
        {
            var blockComments = @"/\*(.*?)\*/";
            var lineComments = @"//(.*?)\r?\n";
            var strings = @"""((\\[^\n]|[^""\n])*)""";
            var verbatimStrings = @"@(""[^""]*"")+";

            return Regex.Replace(inputtext, blockComments + "|" + lineComments + "|" + strings + "|" + verbatimStrings, 
                me => {
                    if (me.Value.StartsWith("/*") || me.Value.StartsWith("//"))
                        return me.Value.StartsWith("//") ? Environment.NewLine : "";
                    return me.Value;
                },
                RegexOptions.Singleline);
        }

        private void InitializeComponent()
        {
            this.btnOK = new System.Windows.Forms.Button();
            this.textBox = new System.Windows.Forms.TextBox();
            this.listCounters = new System.Windows.Forms.ListBox();
            this.label1 = new System.Windows.Forms.Label();
            this.btnAddNumericCounter = new System.Windows.Forms.Button();
            this.SuspendLayout();
            // 
            // btnOK
            // 
            this.btnOK.Location = new System.Drawing.Point(1542, 763);
            this.btnOK.Name = "btnOK";
            this.btnOK.Size = new System.Drawing.Size(193, 50);
            this.btnOK.TabIndex = 1;
            this.btnOK.Text = "OK";
            this.btnOK.UseVisualStyleBackColor = true;
            this.btnOK.Click += new System.EventHandler(this.btnOK_Click);
            // 
            // textBox
            // 
            this.textBox.Font = new System.Drawing.Font("Courier New", 8F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.textBox.Location = new System.Drawing.Point(12, 12);
            this.textBox.Multiline = true;
            this.textBox.Name = "textBox";
            this.textBox.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
            this.textBox.Size = new System.Drawing.Size(1723, 554);
            this.textBox.TabIndex = 2;
            // 
            // listCounters
            // 
            this.listCounters.FormattingEnabled = true;
            this.listCounters.ItemHeight = 29;
            this.listCounters.Location = new System.Drawing.Point(12, 635);
            this.listCounters.Name = "listCounters";
            this.listCounters.ScrollAlwaysVisible = true;
            this.listCounters.Size = new System.Drawing.Size(497, 178);
            this.listCounters.TabIndex = 3;
            // 
            // label1
            // 
            this.label1.AutoSize = true;
            this.label1.Location = new System.Drawing.Point(12, 596);
            this.label1.Name = "label1";
            this.label1.Size = new System.Drawing.Size(110, 29);
            this.label1.TabIndex = 4;
            this.label1.Text = "Counters";
            // 
            // btnAddNumericCounter
            // 
            this.btnAddNumericCounter.Location = new System.Drawing.Point(515, 761);
            this.btnAddNumericCounter.Name = "btnAddNumericCounter";
            this.btnAddNumericCounter.Size = new System.Drawing.Size(347, 52);
            this.btnAddNumericCounter.TabIndex = 5;
            this.btnAddNumericCounter.Text = "Add Numeric Counter";
            this.btnAddNumericCounter.UseVisualStyleBackColor = true;
            this.btnAddNumericCounter.Click += new System.EventHandler(this.btnAddNumericCounter_Click);
            // 
            // EditTitleBlockForm
            // 
            this.ClientSize = new System.Drawing.Size(1747, 828);
            this.Controls.Add(this.btnAddNumericCounter);
            this.Controls.Add(this.label1);
            this.Controls.Add(this.listCounters);
            this.Controls.Add(this.textBox);
            this.Controls.Add(this.btnOK);
            this.Name = "EditTitleBlockForm";
            this.ResumeLayout(false);
            this.PerformLayout();

        }

        private void btnAddNumericCounter_Click(object sender, System.EventArgs e)
        {
            string counterstartvalue = "1";
            SimpleDlg.ShowInputBoxDialog("Counter Start Value", "Counter Start Value", ref counterstartvalue);

            listCounters.Items.Add(string.Format("{0}:{1}",  countCounters++, counterstartvalue) );
        }

        private void btnOK_Click(object sender, System.EventArgs e)
        {
            int maxReqCounter = -1;
            string[] stringSeparators = new string[] { "\r\n" };
            string attText = RemoveComments(textBox.Text);
            string[] attlines = attText.Split(stringSeparators, StringSplitOptions.RemoveEmptyEntries);
            for (int j = 0; j < attlines.Length; j++)
            {
                int idx = attlines[j].IndexOf(' ');
                //string attTag = InsertTitle.Left(attlines[j], idx).Trim();
                string attValue = attlines[j].Substring(idx + 1).Trim();
                int mrc = GetMaxReqCounter(attValue, 0);

                Console.WriteLine(string.Format("{0}:: {1}", attValue, mrc));

                maxReqCounter = Math.Max(maxReqCounter, mrc);
            }

            int nc = listCounters.Items.Count -1;
            
            if(maxReqCounter > nc)
            {
                Autodesk.AutoCAD.ApplicationServices.Core.Application.ShowAlertDialog(
                    string.Format("maxReqCounter= {0}\nnc= {1}\nUsing counters whhich are not defined!", maxReqCounter, nc));
                   
            }
            else
            {
                DialogResult = DialogResult.OK;
            }

        }

        /// <summary>
        /// Limited to 9 counters!
        /// </summary>
        /// <param name="s"></param>
        /// <param name="stidx"></param>
        /// <returns></returns>
        int GetMaxReqCounter(string s, int sidx)
        {
            int idx = s.IndexOf('{', sidx);
            if (idx == -1)
                return -1;
            else
            {
                idx++;
                int mv = (int)(char.GetNumericValue(s[idx]) + 0.5);
                return Math.Max(mv, GetMaxReqCounter(s, idx));
            }
        }
    }

}